/*

Copyright (c) 2009, Barry Watson
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Barry Watson ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Barry Watson BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __KERNEL_H__
#define __KERNEL_H__

#include <kernel/types.h>

typedef enum {
  PERIODIC_JOB, 
  SPORADIC_JOB
} KERN_job_type_t;

typedef struct KERN_job KERN_job_t;
struct KERN_job
{
  KERN_job_type_t type;
  U32 preemption_level;
  U32 relative_deadline;
  U32 absolute_deadline;
  U32 period;
  U32 arrival_time;
  KERN_job_t *next_job;
  KERN_job_t *prev_job;    
  void (*function)(void);
};


typedef struct
{
  U32 available;
  U32 *ceiling;
} KERN_resource_t;

void KERN_init (void);
void KERN_resource_request (KERN_resource_t *res, U32 amount);
void KERN_resource_release (void);
KERN_resource_t *KERN_resource_create (U32 available, U32 *ceiling);
KERN_job_t *KERN_job_create (U32 period, U32 relative_deadline, void (*function)(void), KERN_job_type_t);
void KERN_clock_tick (void);
void KERN_job_arrive (KERN_job_t *job);

#ifdef OPT_USE_HOOKS
void HOOK_job_arrive (KERN_job_t *job);
void HOOK_job_start (KERN_job_t *job);
void HOOK_job_finish (KERN_job_t *job);
#endif


typedef struct KERN_cab_buffer KERN_cab_buffer_t;
struct KERN_cab_buffer
{
  KERN_cab_buffer_t *next;
  U32 use_counter;
  U8 data[1];
};

typedef struct
{
  U32 number_of_buffers;
  KERN_cab_buffer_t *most_recent;
  KERN_cab_buffer_t *free_list;
} KERN_cab_t;

void KERN_cab_create(KERN_cab_t *the_cab, U8 *memory, U32 number_of_buffers, U32 buffer_size);
KERN_cab_buffer_t * KERN_cab_reserve(KERN_cab_t *cab);
KERN_cab_buffer_t * KERN_cab_get(KERN_cab_t *cab);
void KERN_cab_put(KERN_cab_t *cab, KERN_cab_buffer_t *buffer);
void KERN_cab_unget(KERN_cab_t *cab, KERN_cab_buffer_t *buffer);
void * KERN_cab_buffer_data (KERN_cab_buffer_t *buffer);


#endif











