/*

Copyright (c) 2009, Barry Watson
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * The name of Barry Wayson may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Barry Watson ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Barry Watson BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <kernel/kernel.h>
#include "sdl.out.h"

/* 
 * We'll use kprintf (defined in the bsp) instead of printf to 
 * make life easier when we build a binary to run on unix.
 */
void kprintf(char *fmt, ...);

void job1 (void)
{
  U32 *data;
  KERN_cab_buffer_t *buffer;
  
  /*
   * This is how you use a resouce as a mutex.
   * The app_main.sdl file configured the resource called "mutex"
   * to have a count of 1 with 1 available at syatem start.
   * We try to get 1 unit of the mutex.
   */
  KERN_resource_request(mutex, 1);
  kprintf("Hello World from job1\n");
  KERN_resource_release();
  
  /*
   * Receive data from the CAB called temp.
   */
  if(!(buffer = KERN_cab_get(&temp)))
    {
      /* 
       * Nobody sent us data. This is only possible when we call
       * KERN_cab_get before the first call to KERN_cab_put.
       */
      return;
    }
  /*
   * Get a pointer to the data connected to the CAB
   */
  data = (U32*)KERN_cab_buffer_data(buffer);
  
  /* 
   * Process that data
   */
  kprintf("job1 cab = %d\n", *data);

  /*
   * Tell the system we're not using that CAB buffer anymore.
   */
  KERN_cab_unget(&temp, buffer);
}

void job2 (void)
{
    static U32 counter = 1;
    KERN_cab_buffer_t *buffer;
    U32 *data;

    KERN_resource_request(mutex, 1);
    kprintf("Hello World from job2\n");
    KERN_resource_release();

    /*
     * This is how we send data with a CAB
     */

    /*
     * Reserve a buffer from the CAB called temp
     */
    buffer = KERN_cab_reserve(&temp);

    /*
     * Get a pointer to the data area. In this case temp holds 32bit unsigned integers.
     */
    data = (U32 *)KERN_cab_buffer_data(buffer);

    /*
     * Copy in the data to the buffer
     */
    *data = counter++;

    /*
     * Make the buffer available to anybody that wants to receive it.
     */
    KERN_cab_put(&temp, buffer);
}

void sporadic1 (void)
{
  kprintf("Hello World from sporadic1\n");
  
  /* The following line should lead to a deadline overrun */
  for(;;);
}


/*
 * You need to enable the hooks option in the app_main.sdl file
 * to see what these functions do.
 */
void HOOK_job_arrive (KERN_job_t *j)
{
  kprintf("job_arrive %x\n", j);
}
void HOOK_job_start (KERN_job_t *j)
{
  kprintf("job_start %x\n", j);
}
void HOOK_job_finish (KERN_job_t *j)
{
  kprintf("job_finish %x\n", j);    
}

