/*

Copyright (c) 2009, Barry Watson
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Barry Watson ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Barry Watson BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <kernel/types.h>
#include <bsp.x86/internal.h>

#define _6845_BASE	0x3D4
#define VIDEO_BASE	0xB8000
#define VIDEO_END       (VIDEO_BASE + 80 * 25 * sizeof(struct character))
#define GREEN_ON_BLACK  0x2

struct character
{
  U8 ch;
  U8 attribute;
};

static struct character *cursor = (struct character *)VIDEO_BASE;
 
static void bcopy(U8 *from, U8 *to, U32 count)
{
  while(count--)
    {
      *to++ = *from++;
    }
}


static void set_cursor(U32 pos)
{
	outb(_6845_BASE,14);
	outb(_6845_BASE+1,pos >> 8);
	outb(_6845_BASE,15);
	outb(_6845_BASE+1,pos&0xff);
}

static void kputc(U8 c)
{
  struct character *runner;

  switch (c)
    {            
    case '\n':
      cursor = cursor + 80;
    case '\r':
      cursor = cursor - (cursor - (struct character *)VIDEO_BASE) % 80;
      break;
      
    default:
      cursor->ch = c;
      cursor->attribute = GREEN_ON_BLACK;
      cursor++;
      break ;
    }
  
  if(cursor >= (struct character *)VIDEO_END)
    {
      bcopy((U8 *)VIDEO_BASE + 80 * sizeof(struct character),
	    (U8 *)VIDEO_BASE,
	    80 * 24 * sizeof(struct character));
      for(runner = (struct character *)VIDEO_BASE + 80 * 24;
	  runner <= (struct character *)VIDEO_END;
	  runner++)
	{
	  runner->ch = 0x20;
	  runner->attribute = GREEN_ON_BLACK;
	}
      cursor = cursor - 80;
    }
  
  set_cursor((U32)(cursor - (struct character *)VIDEO_BASE));
}


static void printn(U32 n, int b)
{
  U32 a;

  a=n/b;
  if(a)
    {
      printn(a, b);
    }
  kputc("0123456789ABCDEF"[(U32)(n%b)]);
}

void kprintf(char *fmt, unsigned int x1)
{
  char c;
  unsigned int *adx;
  char *s;

  adx = &x1;

 loop:
  while((c=*fmt++) != '%')
    {
      if(c == '\0')
	{
	  return;
	}
      kputc(c);
    }
  c=*fmt++;
  if(c == 'd' || c=='u' || c=='o' || c=='x')
    {
      printn((U32)*adx, c=='o'?8:(c=='x'?16:10));
    }
  else if(c=='s')
    {
      s=(char*)*adx;
      while(*s)
	{
	  c=*s++;
	  kputc(c);
	}
    }
  adx++;
  goto loop;
}


