/*

Copyright (c) 2009, Barry Watson
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Barry Watson ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Barry Watson BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

	.data
	.globl	gdt
	.globl	gdtr
	.globl	idt
	.globl	idtr

gdt:	
	# null descriptor (0x00)	
	.word	0		
	.word	0
	.word	0
	.word	0

	# code segment
 	.word	0xffff		# limit B15 - B0
	.word	0		# base B15 - B0
	.byte	0		# base B32 - B16
	.byte	0x9a		# present code&data read&execute
	.byte	0xcf		# 4K granularity, 32-bit, limit B19 - B16
	.byte	0		# base B31 - B24

	# data segment
	.word	0xffff		# limit B15 - B0
	.word	0		# base B15 - B0
	.byte	0		# base B23 - B16
	.byte	0x92		# present code&data read&write
	.byte	0xcf		# 4K granularity, 32-bit, limit B19 - B16
	.byte	0		# base B31 - B24

	# stack segment
	.word	0xffff		# limit B15 - B0
	.word	0		# base B15 - B0
	.byte	0		# base B23 - B16
	.byte	0x92		# present code&data read&write
	.byte	0xcf		# 4K granularity, 32-bit, limit B19 - B16
	.byte	0		# base B31 - B24

gdt_end	=	.
gdtr:	.word	gdt_end-gdt-1	# sizeof _gdt -1 (in bytes)
	.long	gdt

idt:	.space	384	# must equal NID*8 (384 == 48 vectors)
idtr:	.word	383	# size of _idt -1 (in bytes)
	.long	idt

	.text
	.align 2
	.globl	_end
	.globl	_edata
	.globl	lidt
	.globl	start

###########################################
# This is the entrypoint into the system. #
# The MBR calls us after loading us into  #
# memory.                                 #
###########################################

start:
	movl	$0x400000, %esp	# stack starts at 4 Mbytes
	 
	movl	$_end,%eax	 
	subl	$_edata,%eax	 
	pushl	%eax		# number of bytes in BSS
	pushl	$_edata		# starting location
	call	bzero		# clear the BSS

	movl	%esp,%esi
	movl	%esi,%eax	
	movl	$4096,%ebx 
	pushl	%ebx		# 4 Kbytes
	pushl	%eax		# stack start address 
	call	bzero		# clear the stack (necessary for ANSI C)

	movl	%esi,%esp

	pushl	$0
	popf			# clear the flags register

	movl	%esp,%ebp

	lgdt	gdtr		# load global descriptors
	
	ljmp	$0x8,$hop	# set up the code segment
hop:
	movl	$0x10,%eax	# set up the data segment
	movw	%ax,%ds
	movw	%ax,%es
	movl	$0x18,%eax	# set up the stack segment
	movw	%ax,%ss
	movl	%esp,%ebp
	
	call	KERN_init       # get the show on the road!
	
halt:	jmp    halt		# should never get here

###########################
# Useful library routines #
###########################

# Output a byte to the port at the specified address.
# void outb(int address, int data)
#
	.globl	outb
outb:	movl	4(%esp),%edx
	movl	8(%esp),%eax
	outb	%al,%dx
	ret

# Clear a memory region.
# void bzero (int address, int count)
#

	.globl bzero
bzero:
	pushl	%edi
	movl	8(%esp),%edi
	movl	12(%esp),%ecx
	movb	$0x00,%al
	cld
	rep
	stosb
	popl	%edi
	ret

# Load interrupt descriptor table from idtr.
# void lidt(void)
#
lidt:
	lidt	idtr
	ret
	
################################
# The system exception vectors #
################################
	.align 4	
_Xdivide:
	pushl	$0xdeadbeef
	pushl	$0
	pushal
	jmp	trap
	
_XsingleStep:	
	pushl	$0xdeadbeef
	pushl	$1
	pushal
	jmp	trap

_XNMI:	
	pushl	$0xdeadbeef
	pushl	$2
	pushal
	jmp	trap

_Xbreakpoint:	
	pushl	$0xdeadbeef
	pushl	$3
	pushal
	jmp	trap

_Xoverflow:	
	pushl	$0xdeadbeef
	pushl	$4
	pushal
	jmp	trap

_Xbound:	
	pushl	$0xdeadbeef
	pushl	$5
	pushal
	jmp	trap

_XundefinedOpcode:	
	pushl	$0xdeadbeef
	pushl	$6
	pushal
	jmp	trap

_XcoprocessorNotAvailable:	
	pushl	$0xdeadbeef
	pushl	$7
	pushal
	jmp	trap

_XdoubleFault:	
	pushl	$8
	pushal
	jmp	trap

_XcoprocessorSegmentOverrun:	
	pushl	$0xdeadbeef
	pushl	$9
	pushal
	jmp	trap

_XinvalidTSS:	
	pushl	$10
	pushal
	jmp	trap

_XsegmentNotPresent:	
	pushl	$11
	pushal
	jmp	trap

_XstackSegmentOverrun:	
	pushl	$12
	pushal
	jmp	trap

_XgeneralProtection:	
	pushl	$13
	pushal
	jmp	trap
					
_XpageFault:	
	pushl	$14
	pushal
	jmp	trap

_Xunassigned:	
	pushl	$0xdeadbeef
	pushl	$15
	pushal
	jmp	trap

_XcoprocessorError:	
	pushl	$0xdeadbeef
	pushl	$16
	pushal
	jmp	trap

_XalignmentCheck:	
	pushl	$0xdeadbeef
	pushl	$17
	pushal
	jmp	trap

_XmachineCheck:	
	pushl	$0xdeadbeef
	pushl	$18
	pushal
	jmp	trap

_Xreserved1:	
	pushl	$0xdeadbeef
	pushl	$19
	pushal
	jmp	trap

_Xreserved2:	
	pushl	$0xdeadbeef
	pushl	$20
	pushal
	jmp	trap

_Xreserved3:	
	pushl	$0xdeadbeef
	pushl	$21
	pushal
	jmp	trap

_Xreserved4:	
	pushl	$0xdeadbeef
	pushl	$22
	pushal
	jmp	trap

_Xreserved5:	
	pushl	$0xdeadbeef
	pushl	$23
	pushal
	jmp	trap

_Xreserved6:	
	pushl	$0xdeadbeef
	pushl	$24
	pushal
	jmp	trap

_Xreserved7:	
	pushl	$0xdeadbeef
	pushl	$25
	pushal
	jmp	trap

_Xreserved8:	
	pushl	$0xdeadbeef
	pushl	$26
	pushal
	jmp	trap

_Xreserved9:	
	pushl	$0xdeadbeef
	pushl	$27
	pushal
	jmp	trap

_Xreserved10:	
	pushl	$0xdeadbeef
	pushl	$28
	pushal
	jmp	trap
									
_Xreserved11:	
	pushl	$0xdeadbeef
	pushl	$29
	pushal
	jmp	trap

_Xreserved12:	
	pushl	$0xdeadbeef
	pushl	$30
	pushal
	jmp	trap

_Xreserved13:	
	pushl	$0xdeadbeef
	pushl	$31
	pushal
	jmp	trap

_Xirq1:	
	pushl	$0xdeadbeef
	pushl	$32
	pushal
	jmp	trap

_Xirq2:	
	pushl	$0xdeadbeef
	pushl	$33
	pushal
	jmp	trap

_Xirq3:	
	pushl	$0xdeadbeef
	pushl	$34
	pushal
	jmp	trap
			
_Xirq4:	
	pushl	$0xdeadbeef
	pushl	$35
	pushal
	jmp	trap

_Xirq5:	
	pushl	$0xdeadbeef
	pushl	$36
	pushal
	jmp	trap

_Xirq6:	
	pushl	$0xdeadbeef
	pushl	$37
	pushal
	jmp	trap

_Xirq7:	
	pushl	$0xdeadbeef
	pushl	$38
	pushal
	jmp	trap

_Xirq8:	
	pushl	$0xdeadbeef
	pushl	$39
	pushal
	jmp	trap

_Xirq9:	
	pushl	$0xdeadbeef
	pushl	$40
	pushal
	jmp	trap

_Xirq10:	
	pushl	$0xdeadbeef
	pushl	$41
	pushal
	jmp	trap

_Xirq11:	
	pushl	$0xdeadbeef
	pushl	$42
	pushal
	jmp	trap

_Xirq12:	
	pushl	$0xdeadbeef
	pushl	$43
	pushal
	jmp	trap

_Xirq13:	
	pushl	$0xdeadbeef
	pushl	$44
	pushal
	jmp	trap

_Xirq14:	
	pushl	$0xdeadbeef
	pushl	$45
	pushal
	jmp	trap

_Xirq15:	
	pushl	$0xdeadbeef
	pushl	$46
	pushal
	jmp	trap

_Xirq16:	
	pushl	$0xdeadbeef
	pushl	$47
	pushal
	jmp	trap
									

	.globl	default_vec
	.align 4
default_vec:
		.long	_Xdivide
		.long	_XsingleStep
		.long	_XNMI
		.long	_Xbreakpoint
		.long	_Xoverflow
		.long	_Xbound
		.long	_XundefinedOpcode
		.long	_XcoprocessorNotAvailable
		.long	_XdoubleFault
		.long	_XcoprocessorSegmentOverrun
		.long	_XinvalidTSS
		.long	_XsegmentNotPresent
		.long	_XstackSegmentOverrun
		.long	_XgeneralProtection
		.long	_XpageFault
		.long	_Xunassigned
		.long	_XcoprocessorError
		.long	_XalignmentCheck
		.long	_XmachineCheck
		.long	_Xreserved1
		.long	_Xreserved2
		.long	_Xreserved3
		.long	_Xreserved4
		.long	_Xreserved5
		.long	_Xreserved6
		.long	_Xreserved7
		.long	_Xreserved8
		.long	_Xreserved9
		.long	_Xreserved10
		.long	_Xreserved11
		.long	_Xreserved12
		.long	_Xreserved13
		.long	_Xirq1
		.long	_Xirq2
		.long	_Xirq3
		.long	_Xirq4
		.long	_Xirq5
		.long	_Xirq6
		.long	_Xirq7
		.long	_Xirq8
		.long	_Xirq9
		.long	_Xirq10
		.long	_Xirq11
		.long	_Xirq12
		.long	_Xirq13
		.long	_Xirq14
		.long	_Xirq15
		.long	_Xirq16

	
################
# BSP routines #
################

#include <bsp.x86/8259a.S>			
	.set	ICU1,0x20
	.set	ICU2,0xA0
	.globl  interrrupt_mask

#
# void BSP_enable_interrupts (void)
#
	.globl  BSP_enable_interrupts
BSP_enable_interrupts:
	cli
	movw	interrupt_mask,%ax
	outb	%al,$IMR1
	shrw	$8,%ax
	outb	%al,$IMR2
	sti
	ret


#
# void BSP_disable_interrupts (void)
#
	.globl BSP_disable_interrupts

BSP_disable_interrupts:
	cli
	movb	$0xff,%al
	outb	%al,$ICU1+1	
	outb	%al,$ICU2+1
	ret


	.end

